/*=========================================================================

   Program: ParaView
   Module:  pqMultiColumnHeaderView.h

   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
   All rights reserved.

   ParaView is a free software; you can redistribute it and/or modify it
   under the terms of the ParaView license version 1.2.

   See License_v1.2.txt for the full ParaView license.
   A copy of this license can be obtained by contacting
   Kitware Inc.
   28 Corporate Drive
   Clifton Park, NY 12065
   USA

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

========================================================================*/
#ifndef pqMultiColumnHeaderView_h
#define pqMultiColumnHeaderView_h

#include "pqWidgetsModule.h" // for exports
#include <QHeaderView>

/**
 * @class pqMultiColumnHeaderView
 * @brief QHeaderView that supports showing multiple sections as one.
 *
 * pqMultiColumnHeaderView extents QHeaderView to support showing multiple
 * adjacent sections as a single section. This is useful for showing vector
 * quantities, for example. Instead of each component taking up header space
 * and making it confusing to understand that the various sections are part of
 * the same vector, pqMultiColumnHeaderView can show all those sections under a
 * single banner. It still supports resizing individual sections thus does not
 * inhibit usability.
 *
 * pqMultiColumnHeaderView simply combines adjacent sections with same
 * (non-empty) `QString` value for Qt::`DisplayRole`. This is done by
 * overriding `QHeaderView::paintSection` and custom painting such
 * sections spanning multiple sections.
 */
class PQWIDGETS_EXPORT pqMultiColumnHeaderView : public QHeaderView
{
  Q_OBJECT;
  typedef QHeaderView Superclass;

public:
  pqMultiColumnHeaderView(Qt::Orientation orientation, QWidget* parent = nullptr);
  ~pqMultiColumnHeaderView() override;

protected:
  void paintSection(QPainter* painter, const QRect& rect, int logicalIndex) const override;

private:
  QPair<int, int> sectionSpan(int visualIndex) const;
  QString sectionDisplayText(int logicalIndex) const;

private:
  Q_DISABLE_COPY(pqMultiColumnHeaderView);
};

#endif
