"""This is present for backwards compatibility alone. Scripts are encouraged to use `numpy_support`
from `vtk` package directly as follows:

    from paraview.vtk.util import numpy_support

"""
from __future__ import absolute_import
from .vtk.util.numpy_support import *
