/*=========================================================================

  Program:   ParaView
  Module:    vtkCPPythonScriptPipelineConfig.h

  Copyright (c) Kitware, Inc.
  All rights reserved.
  See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#ifndef vtkCPPythonScriptPipelineConfig_h
#define vtkCPPythonScriptPipelineConfig_h

// These variables are used to locate the python module files since we cannot
// reply on the executable path to locate python module files.
#define PARAVIEW_BINARY_DIR "D:/bld/paraview_1588158201093/work/build"
#define PARAVIEW_INSTALL_DIR "D:/bld/paraview_1588158201093/_h_env"

#endif
