
#ifndef VTKLAGRANGIANPARTICLETRACKER_EXPORT_H
#define VTKLAGRANGIANPARTICLETRACKER_EXPORT_H

#ifdef VTKLAGRANGIANPARTICLETRACKER_STATIC_DEFINE
#  define VTKLAGRANGIANPARTICLETRACKER_EXPORT
#  define VTKLAGRANGIANPARTICLETRACKER_NO_EXPORT
#else
#  ifndef VTKLAGRANGIANPARTICLETRACKER_EXPORT
#    ifdef vtkLagrangianParticleTracker_EXPORTS
        /* We are building this library */
#      define VTKLAGRANGIANPARTICLETRACKER_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKLAGRANGIANPARTICLETRACKER_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKLAGRANGIANPARTICLETRACKER_NO_EXPORT
#    define VTKLAGRANGIANPARTICLETRACKER_NO_EXPORT 
#  endif
#endif

#ifndef VTKLAGRANGIANPARTICLETRACKER_DEPRECATED
#  define VTKLAGRANGIANPARTICLETRACKER_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKLAGRANGIANPARTICLETRACKER_DEPRECATED_EXPORT
#  define VTKLAGRANGIANPARTICLETRACKER_DEPRECATED_EXPORT VTKLAGRANGIANPARTICLETRACKER_EXPORT VTKLAGRANGIANPARTICLETRACKER_DEPRECATED
#endif

#ifndef VTKLAGRANGIANPARTICLETRACKER_DEPRECATED_NO_EXPORT
#  define VTKLAGRANGIANPARTICLETRACKER_DEPRECATED_NO_EXPORT VTKLAGRANGIANPARTICLETRACKER_NO_EXPORT VTKLAGRANGIANPARTICLETRACKER_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKLAGRANGIANPARTICLETRACKER_NO_DEPRECATED
#    define VTKLAGRANGIANPARTICLETRACKER_NO_DEPRECATED
#  endif
#endif

#endif /* VTKLAGRANGIANPARTICLETRACKER_EXPORT_H */
