
#ifndef VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_EXPORT_H
#define VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_EXPORT_H

#ifdef VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_STATIC_DEFINE
#  define VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_EXPORT
#  define VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_NO_EXPORT
#else
#  ifndef VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_EXPORT
#    ifdef BagPlotViewsAndFiltersBagPlot_EXPORTS
        /* We are building this library */
#      define VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_NO_EXPORT
#    define VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_NO_EXPORT 
#  endif
#endif

#ifndef VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_DEPRECATED
#  define VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_DEPRECATED_EXPORT
#  define VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_DEPRECATED_EXPORT VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_EXPORT VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_DEPRECATED
#endif

#ifndef VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_DEPRECATED_NO_EXPORT
#  define VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_DEPRECATED_NO_EXPORT VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_NO_EXPORT VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_NO_DEPRECATED
#    define VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_NO_DEPRECATED
#  endif
#endif

#endif /* VTKBAGPLOTVIEWSANDFILTERSBAGPLOT_EXPORT_H */
